# -*- encoding: utf-8 -*-
require 'spec_helper'
#絵師

describe ArtistsController do
  before do
    FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
  end
  
  describe '一覧表示に於いて' do
    before do
      Artist.stub(:list).and_return([@artist, @artist, @artist])
      sign_in @user
    end
    context '事前チェックする' do
      it '与えられたpageがセットされている' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq Artist.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq Artist.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq Artist.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '絵師モデルに一覧を問い合わせている' do
        Artist.should_receive(:list).exactly(1)
        get :index
      end
      it '@artistsにリストを取得している' do
        get :index
        assigns(:artists).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '絵師モデルにjson一覧出力オプションを問い合わせている' do
          Artist.should_receive(:list_json_opt).exactly(1)
          get :index, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは絵師っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("name").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '閲覧に於いて' do
    before do
      Artist.stub(:show).and_return(@artist)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @artist.id
        response.should be_success
      end
      it '絵師モデルに単体取得を問い合わせている' do
        Artist.should_receive(:show).exactly(1)
        get :show
      end
      it '@artistにアレを取得している' do
        get :show, :id => @artist.id
        assigns(:artist).should eq(@artist)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @artist.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @artist.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '絵師モデルにjson単体出力オプションを問い合わせている' do
          Artist.should_receive(:show_json_opt).exactly(1)
          get :show, :id => @artist.id, :format => :json
        end
        it 'データがアレになっている' do
          get :show, :id => @artist.id, :format => :json
          json = JSON.parse response.body
          json["name"].should match(/yas/)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @artist.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @artist.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @artist.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @artist.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
=begin
    context '対象作家がないとき' do
      context 'html形式' do
        it '例外404 not_foundを返す' do
          lambda{
            get :show, :id => 0
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
      context 'json形式' do
        it '例外404 not_foundを返す' do
          lambda{ 
            get :show, :id => 0, :format => :json
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
    end
=end
  end
  
  describe '絵師数取得に於いて' do
    before do
      Artist.should_receive(:visible_count).and_return(3)
#      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :count, :format => :json
        response.should be_success 
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :count, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがHash構造になっていて絵師数が3である' do
          get :count, :format => :json
          json = JSON.parse response.body
          json["count"].should == 3
        end
      end
    end
  end

  describe '新規作成フォーム表示に於いて' do
    before do
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :new
        response.should be_success 
      end
      it '@artistに新規データを用意している' do
        get :new
        assigns(:artist).should be_a_new(Artist)
      end
      it '絵師モデルにデフォルト値補充を依頼している' do
        Artist.any_instance.should_receive(:supply_default).exactly(1)
        get :new
      end
      context 'html形式' do
        it 'newテンプレートを描画する' do
          get :new
          response.should render_template("new")
        end
      end
      context 'js形式' do
        it 'new.jsテンプレートを描画する' do
          get :new, :format => :js
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :new, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '絵師モデルにjson単体出力オプションを問い合わせている' do
          Artist.should_receive(:show_json_opt).exactly(1)
          get :new, :format => :json
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :new
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :new
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :new, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :new, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '新規作成に於いて' do
    before do
      sign_in @user
      @attr = FactoryGirl.attributes_for(:artist, :author_id => @author.id, :name => 'ken')
    end
    context '事前チェックしておく' do
      it '絵師モデルにデフォルト値補充を依頼している' do
        Artist.any_instance.should_receive(:supply_default).exactly(1)
        post :create, :artist => @attr
      end
      it '絵師モデルにカラム値復元を依頼している' do
        Artist.any_instance.should_receive(:attributes=).exactly(1)
        post :create, :artist => @attr
      end
      it '絵師モデルに上書き補充を依頼している' do
        Artist.any_instance.should_receive(:overwrite).exactly(1)
        post :create, :artist => @attr
      end
      it 'モデルに保存依頼する' do
        Artist.any_instance.should_receive(:save).exactly(1)
        post :create, :artist => @attr
      end
    end
    context 'つつがなく終わるとき' do
      it "@artistに作成された絵師を保持していて、それがDBにある" do
        post :create, :artist => @attr
        assigns(:artist).should be_a(Artist)
        assigns(:artist).should be_persisted
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          Artist.any_instance.stub(:save).and_return(true)
          post :create, :artist => @attr
          response.status.should eq 302
        end
        it '作成された絵師の表示ページへ遷移する' do
#          Artist.any_instance.stub(:save).and_return(true)
          post :create, :artist => @attr
          response.should redirect_to(Artist.last)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
#          Artist.any_instance.stub(:save).and_return(true)
          post :create, :artist => @attr, :format => :json
          response.should be_success 
        end
        it '作成された絵師をjsonデータで返す' do
          post :create, :artist => @attr, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '絵師モデルにjson単体出力オプションを問い合わせている' do
          Artist.should_receive(:show_json_opt).exactly(1)
          post :create, :artist => @attr, :format => :json
        end
        it 'データがアレになっている' do
          post :create, :artist => @attr, :format => :json
          json = JSON.parse response.body
          json["name"].should match(/ken/)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          post :create, :artist => @attr
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          post :create, :artist => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          post :create, :artist => @attr, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          post :create, :artist => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗した' do
      before do
        Artist.any_instance.stub(:save).and_return(false)
      end
      it "未保存の絵師を保持している" do
        post :create, :artist => @attr
        assigns(:artist).should be_a_new(Artist)
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          post :create, :artist => @attr
          response.status.should eq 200
        end
        it '新規ページを描画する' do
          post :create, :artist => @attr
          response.should render_template("new")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          post :create, :artist => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          post :create, :artist => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

  describe '編集フォーム表示に於いて' do
    before do
      sign_in @user
      Artist.stub(:edit).and_return(@artist)
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :edit, :id => @artist.id
        response.should be_success 
      end
      it '絵師モデルに編集取得を問い合わせている' do
        Artist.should_receive(:edit).exactly(1)
        get :edit, :id => @artist.id
      end
      it '@artistにデータを用意している' do
        get :edit, :id => @artist.id
        assigns(:artist).should eq @artist
      end
      context 'html形式' do
        it 'editテンプレートを描画する' do
          get :edit, :id => @artist.id
          response.should render_template("edit")
        end
      end
      context 'js形式' do
        it 'edit.jsテンプレートを描画する' do
          get :edit, :id => @artist.id, :format => :js
          response.should render_template("edit")
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :edit, :id => @artist.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :edit, :id => @artist.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'js形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :edit, :id => @artist.id, :format => :js
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :edit, :id => @artist.id, :format => :js
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end

  describe '更新に於いて' do
    before do
      @attr = FactoryGirl.attributes_for(:artist, :author_id => @author.id, :name => 'ryu')
      sign_in @user
    end
    context '事前チェックしておく' do
      it '絵師モデルに編集取得を問い合わせている' do
        Artist.stub(:edit).with(any_args()).and_return @artist
        Artist.should_receive(:edit).exactly(1)
        put :update, :id => @artist.id, :artist => @attr
      end
      it '絵師モデルにカラム値復元を依頼している' do
        Artist.any_instance.should_receive(:attributes=).exactly(1)
        put :update, :id => @artist.id, :artist => @attr
      end
      it '絵師モデルに上書き補充を依頼している' do
        Artist.any_instance.should_receive(:overwrite).exactly(1)
        put :update, :id => @artist.id, :artist => @attr
      end
      it 'モデルに更新を依頼する' do
        Artist.any_instance.stub(:save).with(any_args).and_return true
        Artist.any_instance.should_receive(:save).exactly(1)
        put :update, :id => @artist.id, :artist => @attr
      end
      it '@artistにアレを取得している' do
        put :update, :id => @artist.id, :artist => @attr
        assigns(:artist).should eq @artist
      end
    end
    context 'つつがなく終わるとき' do
      it '更新される' do
        put :update, :id => @artist.id, :artist => @attr
        Artist.find(@artist.id).name.should eq 'ryu'
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          Artist.any_instance.stub(:save).with(any_args()).and_return(true)
          put :update, :id => @artist.id, :artist => @attr
          response.status.should eq 302
        end
        it '更新された絵師の表示ページへ遷移する' do
          put :update, :id => @artist.id, :artist => @attr
          response.should redirect_to(@artist)
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          Artist.any_instance.stub(:save).with(any_args()).and_return(true)
          put :update, :id => @artist.id, :artist => @attr, :format => :json
          response.should be_success 
        end
        it 'ページ本体は特に返さない' do
          Artist.any_instance.stub(:save).with(any_args()).and_return(true)
          put :update, :id => @artist.id, :artist => @attr, :format => :json
          response.body.should match /./
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      it 'ステータスコード302 Foundを返す' do
        put :update, :id => @artist.id, :artist => @attr
        response.status.should eq 302
      end
      context 'html形式' do
        it 'サインインページへ遷移する' do
          put :update, :id => @artist.id, :artist => @attr
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it '応答メッセージにUnauthorizedを返す' do
          put :update, :id => @artist.id, :artist => @attr, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '検証、保存に失敗したとき' do
      before do
        Artist.any_instance.stub(:save).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード200 Okを返す' do
          put :update, :id => @artist.id, :artist => @attr
          response.status.should eq 200
        end
        it '編集ページを描画する' do
          put :update, :id => @artist.id, :artist => @attr
          response.should render_template("edit")
        end
      end
      context 'json形式' do
        it 'ステータスコード422 unprocessable_entity を返す' do
          Artist.any_instance.stub(:save).and_return(false)
          put :update, :id => @artist.id, :artist => @attr, :format => :json
          response.status.should eq 422
        end
        it '応答メッセージUnprocessable Entityを返す' do
          put :update, :id => @artist.id, :artist => @attr, :format => :json
          response.message.should match(/Unprocessable/)
        end
      end
    end
  end

end
